
/* Chrysalide - Outil d'analyse de fichiers binaires
 * new.c - décompilation des créations de nouvelles instances
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../../../arch/dalvik/operands/pool.h"
#include "../../../decomp/expr/assign.h"
#include "../../../decomp/expr/call.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'new-instance'.            *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_new_instance(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    uint32_t index;                         /* Indice dans la table        */
    GDataType *type;                        /* Type concerné par l'opérat° */
    GBinRoutine *constructor;               /* Constructeur reconstruit    */
    GDecInstruction *call;                  /* Appel au constructeur       */
    GDecInstruction *dest;                  /* Registre de destination     */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    index = g_dalvik_pool_operand_get_index(G_DALVIK_POOL_OPERAND(operand));
    type = get_type_from_dex_pool(G_DEX_FORMAT(g_dec_context_get_format(ctx)), index);

    constructor = g_binary_routine_new_constructor(type);
    call = g_routine_call_new(constructor);

    operand = g_arch_instruction_get_operand(instr, 0);
    dest = g_dec_context_convert_register(ctx, operand, true, addr);

    result = g_assign_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(call));

    return result;

}
