
/* Chrysalide - Outil d'analyse de fichiers binaires
 * context.c - contextes de décodage DEX
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "context.h"


#include "../context-int.h"



/* Contexte de décodage DEX (instance) */
struct _GDexDemangler
{
    GDemanglingContext parent;              /* A laisser en premier        */

};

/* Contexte de décodage DEX (classe) */
struct _GDexDemanglerClass
{
    GDemanglingContextClass parent;         /* A laisser en premier        */

};


/* Initialise la classe des contextes de décodage DEX. */
static void g_dex_demangler_class_init(GDexDemanglerClass *);

/* Initialise une instance de contexte pour décodage DEX. */
static void g_dex_demangler_init(GDexDemangler *);

/* Supprime toutes les références externes. */
static void g_dex_demangler_dispose(GDexDemangler *);

/* Procède à la libération totale de la mémoire. */
static void g_dex_demangler_finalize(GDexDemangler *);


/* Procède au décodage d'une chaîne de caractères. */
extern bool demangle_dex_routine(GDexDemangler *, const char *);

/* Procède au décodage d'une chaîne de caractères. */
extern bool demangle_dex_type(GDexDemangler *, const char *);



/* Indique le type défini pour un contexte de décodage DEX. */
G_DEFINE_TYPE(GDexDemangler, g_dex_demangler, G_TYPE_DEMANGLING_CONTEXT);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des contextes de décodage DEX.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dex_demangler_class_init(GDexDemanglerClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GDemanglingContextClass *context;       /* Version parente             */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_dex_demangler_dispose;
    object->finalize = (GObjectFinalizeFunc)g_dex_demangler_finalize;

    context = G_DEMANGLING_CONTEXT_CLASS(klass);

    context->demangle_type = (demangle_fc)demangle_dex_type;
    context->demangle_routine = (demangle_fc)demangle_dex_routine;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : demangler = instance à initialiser.                          *
*                                                                             *
*  Description : Initialise une instance de contexte pour décodage DEX.       *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dex_demangler_init(GDexDemangler *demangler)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : demangler = instance d'objet GLib à traiter.                 *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dex_demangler_dispose(GDexDemangler *demangler)
{
    G_OBJECT_CLASS(g_dex_demangler_parent_class)->dispose(G_OBJECT(demangler));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : demangler = instance d'objet GLib à traiter.                 *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dex_demangler_finalize(GDexDemangler *demangler)
{
    G_OBJECT_CLASS(g_dex_demangler_parent_class)->finalize(G_OBJECT(demangler));

}
