
/* Chrysalide - Outil d'analyse de fichiers binaires
 * manager.h - prototypes pour la prise en compte d'une syntaxe du langage d'assemblage
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _TOOLS_D2C_SYNTAX_MANAGER_H
#define _TOOLS_D2C_SYNTAX_MANAGER_H


#include <stdbool.h>


#include "../pproc.h"
#include "../bits/manager.h"
#include "../conv/manager.h"



/* Type d'éléments de syntaxe */
typedef enum _SyntaxItemType
{
    SIT_KEYWORD,                            /* Elément de l'instruction    */
    SIT_INT_OPERAND,                        /* Propriété d'architecture    */
    SIT_EXT_OPERAND                         /* Opérande généraliste        */

} SyntaxItemType;

/* Syntaxe d'une ligne d'assembleur */
typedef struct _asm_syntax asm_syntax;


/* Crée un nouvel indicateur pour l'écriture d'une instruction. */
asm_syntax *create_asm_syntax(void);

/* Supprime de la mémoire un indicateur d'écriture ASM. */
void delete_asm_syntax(asm_syntax *);

/* Enregistre la présence d'un nouvel opérande dans la syntaxe. */
void register_syntax_item(asm_syntax *, char *, SyntaxItemType);

/* Marque les champs de bits effectivement utilisés. */
bool mark_syntax_items(const asm_syntax *, const coding_bits *, const conv_list *);

/* Déclare les variables C associées aux opérandes de syntaxe. */
bool declare_syntax_items(const asm_syntax *, int, const coding_bits *, const conv_list *, unsigned int);

/* Fournit si elle existe un nom nouveau pour une instruction. */
const char *get_new_keyword_from_syntax_items(const asm_syntax *);

/* Définit les variables C associées aux opérandes de syntaxe. */
bool define_syntax_items(const asm_syntax *, int, const char *, const coding_bits *, const conv_list *, const pre_processor *, bool *);



#endif  /* _TOOLS_D2C_SYNTAX_MANAGER_H */
